@@ -49,6 +49,8 @@
 #include <osreldate.h>
 #include <sys/sysctl.h>
 
+#include <dlfcn.h>
+
 
 #if __FreeBSD_version < 400017
 
@@ -86,7 +88,7 @@
 #endif
 
 
-#if (NGX_HAVE_FILE_AIO || NGX_HAVE_AIO)
+#if (NGX_HAVE_FILE_AIO)
 #include <aio.h>
 typedef struct aiocb  ngx_aiocb_t;
 #endif
@@ -100,12 +102,6 @@ typedef struct aiocb  ngx_aiocb_t;
 #endif
 
 
-#if (__FreeBSD_version < 430000 || __FreeBSD_version < 500012)
-
-pid_t rfork_thread(int flags, void *stack, int (*func)(void *arg), void *arg);
-
-#endif
-
 #ifndef IOV_MAX
 #define IOV_MAX   1024
 #endif
